==> dna-0.2.0.tar.gz: lib/coder.dart <==
part of dna;

TypedData encode(dynamic object) {
  var totalSize = 0;
  struct(object, (field, type, fieldMirror){
    var size = DynamicLibrary.argTypeToSize(type);
    if (totalSize % size != 0) {
      totalSize += (size - totalSize % size);
    }
    totalSize += size;
  });

  var offset = 0;
  TypedData data = new Uint8List(totalSize);
  ByteData view = new ByteData.view(data.buffer);
  struct(object, (field, type, fieldMirror){
    var size = DynamicLibrary.argTypeToSize(type);
    switch (type){
      case ARG_CHAR:
      case ARG_UCHAR:
      case ARG_SHORT:
      case ARG_USHORT:
      case ARG_INT:
      case ARG_UINT:
      case ARG_LONG:
      case ARG_ULONG:
      case ARG_LONGLONG:
      case ARG_ULONGLONG:
      case ARG_POINTER:
        setInt(view, size, offset, field != null ? field : 0);
        break;
      case ARG_BOOL:
        setInt(view, size, offset, field ? 1 : 0);
        break;
      case ARG_FLOAT:
      case ARG_DOUBLE:
        setFloat(view, size, offset, field != null ? field : 0.0);
        break;
      case ARG_STRING:
        String str = field;
        for (var char in str.codeUnits){
          setInt(view, 1, offset, char);
          offset++;
        }
        break;
    }
    if (offset % size != 0) {
      offset += (size - offset % size);
    }
    offset += size;
  });

  //print(data);
  return data;
}

dynamic decode(TypedData data, dynamic object) {
  //print(data);

  ByteData view = new ByteData.view(data.buffer);
  InstanceMirror classMirror = reflect(object);

  var offset = 0;
  struct(object, (field, type, fieldMirror) {
    var size = DynamicLibrary.argTypeToSize(type);
    dynamic value;

    switch (type){
      case ARG_CHAR:
      case ARG_UCHAR:
      case ARG_SHORT:
      case ARG_USHORT:
      case ARG_INT:
      case ARG_UINT:
      case ARG_LONG:
      case ARG_ULONG:
      case ARG_LONGLONG:
      case ARG_ULONGLONG:
      case ARG_POINTER:
        value = getInt(view, size, offset);
        break;
      case ARG_BOOL:
        value = getInt(view, size, offset);
        break;
      case ARG_FLOAT:
      case ARG_DOUBLE:
        value = getFloat(view, size, offset);
        break;
      case ARG_STRING:
        value = "";
        break;
    }

    classMirror.setField(fieldMirror.simpleName, value);

    if (offset % size != 0) {
      offset += (size - offset % size);
    }
    offset += size;
  });
  return object;
}

int getInt(ByteData view, int size, int offset){
  switch (size){
    case 1:
      return view.getInt8(offset);
    case 2:
      return view.getInt16(offset, Endianness.LITTLE_ENDIAN);
    case 4:
      return view.getInt32(offset, Endianness.LITTLE_ENDIAN);
    case 8:
      return view.getInt64(offset, Endianness.LITTLE_ENDIAN);
  }
}

double getFloat(ByteData view, int size, int offset){
  switch (size){
    case 4:
      return view.getFloat32(offset);
    case 8:
      return view.getFloat64(offset);
  }
}

void setInt(ByteData view, int size, int offset, int value){
  switch (size){
    case 1:
      view.setInt8(offset, value);
      break;
    case 2:
      view.setInt16(offset, value);
      break;
    case 4:
      view.setInt32(offset, value);
      break;
    case 8:
      view.setInt64(offset, value);
      break;
  }
}

void setFloat(ByteData view, int size, int offset, double value){
  switch (size){
    case 4:
      view.setFloat32(offset, value);
      break;
    case 8:
      view.setFloat64(offset, value);
      break;
  }
}


int struct(dynamic param, dynamic process(dynamic field, int type, DeclarationMirror fieldMirror)){
  ClassMirror classMirror = reflect(param).type;
  //print(classMirror);

  var fields = classMirror.declarations.values.where((d)=>true);

  for (var fieldMirror in fields){
    var annot = Util.declAnnot(fieldMirror, Field);
    if (annot == null) continue;
    var argType = annot != null ? annot.getField(#type).reflectee : DynamicLibrary.typeToArgType(fieldMirror.runtimeType);
    var field = reflect(param).getField(fieldMirror.simpleName).reflectee;

    process(field, argType, fieldMirror);
  }
}

==> dna-0.2.1.tar.gz: lib/coder.dart <==
// Copyright (c) 2016, root ext. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

part of dna;

TypedData encode(dynamic object) {
  var totalSize = 0;
  struct(object, (field, type, fieldMirror){
    var size = DynamicLibrary.ctypeToSize(type);
    if (totalSize % size != 0) {
      totalSize += (size - totalSize % size);
    }
    totalSize += size;
  });

  var offset = 0;
  TypedData data = new Uint8List(totalSize);
  ByteData view = new ByteData.view(data.buffer);
  struct(object, (field, type, fieldMirror){
    var size = DynamicLibrary.ctypeToSize(type);
    switch (type){
      case C_CHAR:
      case C_UCHAR:
      case C_SHORT:
      case C_USHORT:
      case C_INT:
      case C_UINT:
      case C_LONG:
      case C_ULONG:
      case C_LONGLONG:
      case C_ULONGLONG:
      case C_POINTER:
        setInt(view, size, offset, field != null ? field : 0);
        break;
      case C_BOOL:
        setInt(view, size, offset, field ? 1 : 0);
        break;
      case C_FLOAT:
      case C_DOUBLE:
        setFloat(view, size, offset, field != null ? field : 0.0);
        break;
      case C_STRING:
        String str = field;
        for (var char in str.codeUnits){
          setInt(view, 1, offset, char);
          offset++;
        }
        break;
    }
    if (offset % size != 0) {
      offset += (size - offset % size);
    }
    offset += size;
  });

  //print(data);
  return data;
}

dynamic decode(TypedData data, dynamic object) {
  //print(data);

  ByteData view = new ByteData.view(data.buffer);
  InstanceMirror classMirror = reflect(object);

  var offset = 0;
  struct(object, (field, type, fieldMirror) {
    var size = DynamicLibrary.ctypeToSize(type);
    dynamic value;

    switch (type){
      case C_CHAR:
      case C_UCHAR:
      case C_SHORT:
      case C_USHORT:
      case C_INT:
      case C_UINT:
      case C_LONG:
      case C_ULONG:
      case C_LONGLONG:
      case C_ULONGLONG:
      case C_POINTER:
        value = getInt(view, size, offset);
        break;
      case C_BOOL:
        value = getInt(view, size, offset);
        break;
      case C_FLOAT:
      case C_DOUBLE:
        value = getFloat(view, size, offset);
        break;
      case C_STRING:
        value = "";
        break;
    }

    classMirror.setField(fieldMirror.simpleName, value);

    if (offset % size != 0) {
      offset += (size - offset % size);
    }
    offset += size;
  });
  return object;
}

int getInt(ByteData view, int size, int offset){
  switch (size){
    case 1:
      return view.getInt8(offset);
    case 2:
      return view.getInt16(offset, Endianness.LITTLE_ENDIAN);
    case 4:
      return view.getInt32(offset, Endianness.LITTLE_ENDIAN);
    case 8:
      return view.getInt64(offset, Endianness.LITTLE_ENDIAN);
  }
}

double getFloat(ByteData view, int size, int offset){
  switch (size){
    case 4:
      return view.getFloat32(offset);
    case 8:
      return view.getFloat64(offset);
  }
}

void setInt(ByteData view, int size, int offset, int value){
  switch (size){
    case 1:
      view.setInt8(offset, value);
      break;
    case 2:
      view.setInt16(offset, value);
      break;
    case 4:
      view.setInt32(offset, value);
      break;
    case 8:
      view.setInt64(offset, value);
      break;
  }
}

void setFloat(ByteData view, int size, int offset, double value){
  switch (size){
    case 4:
      view.setFloat32(offset, value);
      break;
    case 8:
      view.setFloat64(offset, value);
      break;
  }
}


int struct(dynamic param, dynamic process(dynamic field, int type, DeclarationMirror fieldMirror)){
  ClassMirror classMirror = reflect(param).type;
  //print(classMirror);

  var fields = classMirror.declarations.values.where((d)=>true);

  for (var fieldMirror in fields){
    var fieldTag = Util.declAnnot(fieldMirror, Field);
    if (fieldTag == null) continue;
    var fieldType = fieldTag != null ? fieldTag.getField(#type).reflectee : DynamicLibrary.dartTypeToCtype(fieldMirror.runtimeType);
    var field = reflect(param).getField(fieldMirror.simpleName).reflectee;

    process(field, fieldType, fieldMirror);
  }
}
